#include <iostream>
#include <cmath>
#include <vector>

using namespace std;

void gaussianElimination(int n, vector<vector<double>>& matrix, vector<double>& vec, vector<double>& result) {
    // Прямий хід
    for (int i = 0; i < n - 1; i++) {
        for (int j = i + 1; j < n; j++) {
            double factor = matrix[j][i] / matrix[i][i];
            for (int k = i; k < n; k++) {
                matrix[j][k] -= factor * matrix[i][k];
            }
            vec[j] -= factor * vec[i];
        }
    }
    
    // Зворотний хід
    for (int i = n - 1; i >= 0; i--) {
        double sum = 0;
        for (int j = i + 1; j < n; j++) {
            sum += matrix[i][j] * result[j];
        }
        result[i] = (vec[i] - sum) / matrix[i][i];
    }
}

void regression(const vector<double>& x, const vector<double>& y, int n, int m) {
    vector<double> coefficients(m + 1);
    
    // Обчислення коефіцієнтів
    for (int j = 0; j <= m; j++) {
        coefficients[j] = 0;
        for (int k = 0; k <= m; k++) {
            double sum_x_power = 0;
            double sum_x_power_y = 0;
            for (int i = 0; i < n; i++) {
                sum_x_power += pow(x[i], j + k);
                sum_x_power_y += y[i] * pow(x[i], k);
            }
            coefficients[j] += sum_x_power_y;
        }
    }
    
    // Обчислення за допомогою системи рівнянь
    vector<vector<double>> matrix(m + 1, vector<double>(m + 1, 0));
    vector<double> vec(m + 1, 0);
    for (int i = 0; i <= m; i++) {
        for (int j = 0; j <= m; j++) {
            matrix[i][j] = 0;
            for (int k = 0; k < n; k++) {
                matrix[i][j] += pow(x[k], i + j);
            }
        }
        vec[i] = 0;
        for (int k = 0; k < n; k++) {
            vec[i] += y[k] * pow(x[k], i);
        }
    }
    vector<double> result(m + 1);
    gaussianElimination(m + 1, matrix, vec, result);
    
    // Вивід результатів
    cout << "Regression (m = " << m << "):\n";
    for (int j = 0; j <= m; j++) {
        cout << "Coefficient a" << j << " = " << result[j] << "\n";
    }
    cout << "\n";
}

int main() {
    vector<double> x = {0.41, 0.46, 0.52, 0.61, 0.66, 0.73};
    vector<double> y = {2.73, 2.31, 1.97, 1.76, 1.53, 1.31};
    int n = x.size(); // Отримання розміру масиву x
    
    regression(x, y, n, 1); // Лінійна апроксимація
    regression(x, y, n, 2); // Квадратична апроксимація
    regression(x, y, n, 3); // Кубічна апроксимація
    
    return 0;
}
